﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using MySql.Data.MySqlClient;
using System.Data;
using System.Collections;

namespace Snmp2Mysql
{
    class DatabaseLinkException : Exception
    {
        public DatabaseLinkException(string msg) : base(msg) { }
        public DatabaseLinkException(string msg, Exception rootCause) : base(msg, rootCause) { }
    }


    abstract class DatabaseConnectionHelper : IDisposable
    {
        protected string mDbConnStr;
        protected MySqlConnection mConn;

        public DatabaseConnectionHelper(string dbConnStr)
        {
            mDbConnStr = dbConnStr;

            try
            {
                mConn = new MySqlConnection(mDbConnStr);
                mConn.Open();
                if (mConn.State != ConnectionState.Open)
                {
                    throw new DatabaseLinkException("unable to open database connection");
                }
            }
            catch (MySql.Data.MySqlClient.MySqlException ex)
            {
                Console.WriteLine("MySQL Exception: {0}", ex.Message);
                throw new DatabaseLinkException("failed to connect", ex);
            }
        }

        public void Close()
        {
            if (mConn != null)
            {
                mConn.Close();
                mConn = null;
            }
        }

        public void Dispose()
        {
            Close();
        }

        ~DatabaseConnectionHelper()
        {
            Close();
        }

    }


    class DeviceDataItemTuple
    {
        private int mId;
        private string mOid;
		private string mDescription;

		public DeviceDataItemTuple(int id, string oid, string description)
        {
            mId = id;
            mOid = oid;
			mDescription = description;
        }

        public int Id
        {
            get { return mId; }
        }

        public string Oid
        {
            get { return mOid; }
        }

		public string Description {
			get { return mDescription; }
		}
    }

    class DeviceDataItemProvider : DatabaseConnectionHelper, IEnumerable<DeviceDataItemTuple>
    {
        private int mDevice;

        public DeviceDataItemProvider(string dbConnStr, int device) : base(dbConnStr)
        {
            mDevice = device;
        }

        IEnumerator IEnumerable.GetEnumerator()
        {
            throw new Exception("not supported");
        }

        IEnumerator<DeviceDataItemTuple> IEnumerable<DeviceDataItemTuple>.GetEnumerator()
        {
            string stmt = "SELECT dedi.id, di.oid, di.description FROM devicedataitem_t dedi, dataitem_t di WHERE dedi.device = " + mDevice + " AND dedi.dataitem = di.id";
            MySqlCommand cmd = new MySqlCommand(stmt, mConn);

            using (MySqlDataReader reader = cmd.ExecuteReader())
            {
                while (reader.Read())
                {
                    // Console.WriteLine("{0} {1}", reader[0], reader[1]);
					DeviceDataItemTuple r = new DeviceDataItemTuple((int)reader[0], (string)reader[1], (string)reader[2]);
                    yield return r;
                }
            }
        }

    }


    class DeviceTuple
    {
        private DeviceDataItemProvider mDeviceDataItemProvider;
        private string mDeviceAddress;
        private string mCommunity;
		private string mDescription;

		public DeviceTuple(DeviceDataItemProvider deviceDataItemProvider, string deviceAddress, string community, string description)
        {
            mDeviceDataItemProvider = deviceDataItemProvider;
            mDeviceAddress = deviceAddress;
            mCommunity = community;
			mDescription = description;
        }

        public DeviceDataItemProvider DeviceDataItemProvider
        {
            get { return mDeviceDataItemProvider; }
        }

        public string DeviceAddress
        {
            get { return mDeviceAddress; }
        }

        public string Community
        {
            get { return mCommunity; }
        }

		public string Description {
			get { return mDescription; }
		}
    }



    class DeviceProvider : DatabaseConnectionHelper, IEnumerable<DeviceTuple>
    {
        public DeviceProvider(string dbConnStr) : base(dbConnStr) { }

        IEnumerator IEnumerable.GetEnumerator()
        {
            throw new Exception("not supported");
        }

        IEnumerator<DeviceTuple> IEnumerable<DeviceTuple>.GetEnumerator()
        {
            string stmt = "SELECT id, address, community, description FROM device_t";
            MySqlCommand cmd = new MySqlCommand(stmt, mConn);

            using (MySqlDataReader reader = cmd.ExecuteReader())
            {
                while (reader.Read())
                {
                    // Console.WriteLine("{0} {1} {2}", reader[0], reader[1], reader[2]);
                    int deviceId = (int)reader[0];
                    string deviceAddress = (string)reader[1];
                    string community = (string)reader[2];
					string description = (string)reader[3];
                    DeviceDataItemProvider deviceDataItemProvider = new DeviceDataItemProvider(mDbConnStr, deviceId);
                    DeviceTuple r = new DeviceTuple(deviceDataItemProvider, deviceAddress, community, description);
                    yield return r;
                }
            }
        }
    }


    class DataCollector : DatabaseConnectionHelper
    {
        public DataCollector(string dbConnStr) : base(dbConnStr) { }

        public void add(int id, string value)
        {
            string stmt = "INSERT INTO collecteddata_t (devicedataitem, value) VALUES(" + id + ", '" + value + "')";
            MySqlCommand cmd = new MySqlCommand(stmt, mConn);
            cmd.ExecuteNonQuery();
        }

    }


    class DatabaseLink
    {
        private string mDbConnStr;

        public DatabaseLink(string dbConnStr)
        {
            mDbConnStr = dbConnStr;
        }

        public DeviceProvider DeviceProvider
        {
            get { return new DeviceProvider(mDbConnStr); }
        }

        public DataCollector DataCollector
        {
            get { return new DataCollector(mDbConnStr);  }
        }
    }
}
