﻿using SnmpSharpNet;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading.Tasks;

namespace Snmp2Mysql
{
    class SnmpGetterException : Exception
    {
        public SnmpGetterException(string msg) : base(msg) {}
        public SnmpGetterException(string msg, Exception rootCause) : base(msg, rootCause) { }
    }

    class SnmpGetterResult
    {
        private Object mIndex;
        private string mValue;
        private string mType;
        private string mOid;

        public SnmpGetterResult(Object index, string value, string type, string oid) 
        {
            mIndex = index;
            mValue = value;
            mType = type;
            mOid = oid;
        }

        public Object Index
        {
            get { return mIndex;  }
        }
        public string Value
        {
            get { return mValue; }
        }
        public string Type
        {
            get { return mType; }
        }
        public string Oid
        {
            get { return mOid; }
        }
    }

    class SnmpGetterResultProvider : IEnumerable<SnmpGetterResult>
    {
        private ArrayList mIndexes;
        private SnmpV2Packet mPacket;

        public SnmpGetterResultProvider(ArrayList indexes, SnmpV2Packet packet)
        {
            mIndexes = indexes;
            mPacket = packet;
        }

        IEnumerator<SnmpGetterResult> IEnumerable<SnmpGetterResult>.GetEnumerator()
        {
            int idx = 0;

            while (true)
            {
                if (idx == mIndexes.Count)
                {
                    yield break;
                }
                else
                {
                    SnmpGetterResult r = new SnmpGetterResult(mIndexes[idx],
                                                              mPacket.Pdu.VbList[idx].Value.ToString(),
                                                              SnmpConstants.GetTypeName(mPacket.Pdu.VbList[idx].Value.Type),
                                                              mPacket.Pdu.VbList[idx].Oid.ToString());
                    idx++;
                    yield return r;
                }
            }
        }

        IEnumerator IEnumerable.GetEnumerator()
        {
            throw new Exception("not supported");
        }

    }

    class SnmpGetter : IDisposable
    {
        private OctetString mCommunity;
        private IpAddress mHost;
        private AgentParameters mParams;
        private UdpTarget mTarget = null;
        private Pdu mPdu;
        private ArrayList mVbIndexes;
        private SnmpV2Packet mPacket;

        public SnmpGetter(string community, string ipAddress)
        {
            mCommunity = new OctetString(community);
            mHost = new IpAddress(ipAddress);
            mParams = new AgentParameters(mCommunity);
            mParams.Version = SnmpVersion.Ver2;
            mTarget = new UdpTarget((IPAddress)mHost, 161, 2000, 1);
            mPdu = new Pdu(PduType.Get);
            mVbIndexes = new ArrayList();
        }

        public void Close() 
        {
            if (mTarget != null)
            {
                mTarget.Close();
                mTarget = null;
            }
        }

        public void Dispose() 
        {
            Close();
        }

        ~SnmpGetter()
        {
            Close();
        }

        public void AddOid(Object index, string oid)
        {
            mVbIndexes.Add(index);
            mPdu.VbList.Add(oid);
        }

        public SnmpGetterResultProvider Exec()
        {
            try
            {
                mPacket = (SnmpV2Packet)mTarget.Request(mPdu, mParams);
            }
            catch (SnmpException e)
            {
                throw new SnmpGetterException(e.Message, e);
            }

            if (mPacket != null)
            {
                if (mPacket.Pdu.ErrorStatus != 0)
                {
                    string msg = String.Format("Error in SNMP reply. Error {0} index {1}",
                        mPacket.Pdu.ErrorStatus,
                        mPacket.Pdu.ErrorIndex);
                    Close();
                    throw new SnmpGetterException(msg);
                } 
                else
                {
                    return new SnmpGetterResultProvider(mVbIndexes, mPacket);
                }
            }
            else
            {
                Close();
                throw new SnmpGetterException("No response received from SNMP agent");
            }
        }



    }
}
